@@ -2,6 +2,7 @@ use crate::linspace::{Lerp, LinSpace, Linear};
 use core::ops::Range;
 use num_traits::real::Real;
 
+/// Iterator returned by [`arange`]
 pub type Arange<T> = LinSpace<T>;
 
 /// Create a new iterator over the range, stepping by `step` each time
@@ -23,7 +24,9 @@ where
     range.into_arange(step)
 }
 
+/// Used by [`arange`]
 pub trait IntoArange<F> {
+    /// Convert self into an [`Arange`]
     fn into_arange(self, step: F) -> Arange<F>;
 }
 
@@ -53,6 +56,6 @@ mod tests {
     #[test]
     fn test_arange() {
         let it = arange(0.0..2.0, 0.5);
-        assert_eq_iter!(it, [0.0, 0.5, 1.0, 1.5]);
+        itertools::assert_equal(it, [0.0, 0.5, 1.0, 1.5]);
     }
 }
